﻿using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Digests;
using Org.BouncyCastle.Crypto.Encodings;
using Org.BouncyCastle.Crypto.Engines;
using Org.BouncyCastle.OpenSsl;
using Org.BouncyCastle.Security;
using System.Text;

internal class Program
{
    static void Main(string[] args)
    {
        var pemReader = new PemReader(File.OpenText(@"key.pem"));
        var rsaKey = pemReader.ReadObject() as Org.BouncyCastle.Crypto.AsymmetricCipherKeyPair;
        if (rsaKey == null)
        {
            return;
        }

        {
            byte[] encrypted = EncryptData2(rsaKey.Public, "test is good");

            Console.WriteLine(DecryptData1(rsaKey.Private, encrypted));
            Console.WriteLine(DecryptData2(rsaKey.Private, encrypted));
        }

        {
            byte[] encrypted = EncryptData1(rsaKey.Public, "test is good");

            Console.WriteLine(DecryptData1(rsaKey.Private, encrypted));
            Console.WriteLine(DecryptData2(rsaKey.Private, encrypted));
        }
    }

    private static byte[] EncryptData1(ICipherParameters keyParam, string data)
    {
        var cipher = CipherUtilities.GetCipher("RSA/NONE/OAEPWITHSHA256ANDMGF1PADDING");
        cipher.Init(true, keyParam);

        {
            byte[] plain = Encoding.UTF8.GetBytes(data);
            byte[] encrypted = new byte[cipher.GetOutputSize(plain.Length)];
            int outputLength = cipher.ProcessBytes(plain, 0, plain.Length, encrypted, 0);
            cipher.DoFinal(encrypted, outputLength);
            return encrypted;
        }
        {
            // Or, simply call this.
            // byte[] plain = Encoding.UTF8.GetBytes(data);
            // return cipher.DoFinal(plain);
        }
    }

    private static byte[] EncryptData2(ICipherParameters keyParam, string data)
    {
        var encrypter = new OaepEncoding(new RsaBlindedEngine(), new Sha256Digest(), new Sha256Digest(), null);
        encrypter.Init(true, keyParam);

        byte[] plain = Encoding.UTF8.GetBytes(data);
        return encrypter.ProcessBlock(plain, 0, plain.Length);
    }

    private static string DecryptData1(ICipherParameters keyParam, byte [] encrypted)
    {
        var cipher = CipherUtilities.GetCipher("RSA/NONE/OAEPWITHSHA256ANDMGF1PADDING");
        cipher.Init(false, keyParam);

        {
            byte[] decrypted = new byte[cipher.GetOutputSize(encrypted.Length)];
            int outputLength = cipher.ProcessBytes(encrypted, 0, encrypted.Length, decrypted, 0);
            cipher.DoFinal(decrypted, outputLength);
            return Encoding.UTF8.GetString(decrypted);
        }
        {
            // Or, simply call this.
            // return Encoding.UTF8.GetString(cipher.DoFinal(encrypted));
        }
    }

    private static string DecryptData2(ICipherParameters keyParam, byte[] encrypted)
    {
        var decrypter = new OaepEncoding(new RsaBlindedEngine(), new Sha256Digest(), new Sha256Digest(), null);
        decrypter.Init(false, keyParam);

        byte [] decrypted = decrypter.ProcessBlock(encrypted, 0, encrypted.Length);
        return Encoding.UTF8.GetString(decrypted);
    }
}
